import os
import sys
import json
from time import sleep
from threading import Thread
from datetime import datetime, timedelta

import requests
from flask import Blueprint, request, jsonify
import paho.mqtt.publish as mqtt_publish


blueprint = Blueprint('detection', __name__)


def get_sunset() -> datetime:
    IPGEOLOCATION_API_KEY = os.environ['IPGEOLOCATION_API_KEY']
    ipgeolocation_api_response = requests.get(f'https://api.ipgeolocation.io/astronomy?apiKey={IPGEOLOCATION_API_KEY}&location=Winter+Haven,+FL')
    ipgeolocation_api_response.raise_for_status()

    astronomical_json = json.loads(ipgeolocation_api_response.content)
    sunset_time = datetime.strptime(astronomical_json['sunset'], '%H:%M').time()
    sunset_date = datetime.now().date()
    if sunset_time < datetime.now().time():
        # Sunset has already passed today
        sunset_date += timedelta(days=1)
    return datetime.combine(sunset_date, sunset_time)


def reset_all_camera_detection_at_sunset():
    while True:
        sunset = get_sunset()
        print(f'Waiting until {sunset} to reset detection for all cameras...', file=sys.stderr)
        seconds_until_sunset = (sunset - datetime.now()).total_seconds()
        sleep(seconds_until_sunset)

        set_camera_detection('front_door', True)
        set_camera_detection('back_door', True)
        set_camera_detection('garage', True)
        set_camera_detection('family_room', True)


def set_camera_detection(camera: str, value: bool, delay: int = 0) -> None:
    sleep(delay)

    mqtt_publish.single(f'frigate/{camera}/detect/set', 'ON' if value else 'OFF', hostname='mqtt', port=1883, auth={'username': os.environ['MQTT_USERNAME'], 'password': os.environ['MQTT_PASSWORD']})


@blueprint.route('/camera/<string:camera>/detect', methods=['POST'])
def camera_detect_POST(camera):
    if not request.json:
        return jsonify({
            'status': 'failure',
            'description': 'Request body needs to be in JSON format'
        }), 400

    value = request.json.get('value', True)
    set_camera_detection(camera, value)

    duration = request.json.get('duration', 0)
    if duration > 0:
        # Start sleeping thread to revert value after duration
        thread = Thread(target=set_camera_detection, args=(camera, not value, duration * 60))
        thread.start()

    return jsonify({
        'status': 'success'
    }), 200


sunset_reset_thread = Thread(target=reset_all_camera_detection_at_sunset, args=())
sunset_reset_thread.start()
